///////////////////////////////////////////////////////////////////////////////////////////////
// Cyclic Redundency Check utilizing x^16 + x^12 + x^5 + x^0 (0x8408)						 //
// Utilizing "C" Bit Shift Implimentation of this CRC										 //
// Author: Unknown																			 //
// Copywrite: General Public Liscense														 //
// Company Modifier: Future Concepts														 //
// Employee Modifier: Michael Leonardi														 //
// Modify Date: 08/03/2006																	 //
// Project: VidPak Ver (G3.0+), Tahoe, Crown, and many more to come...						 //
///////////////////////////////////////////////////////////////////////////////////////////////
// This code was pulled from Crystal Fontz CFA635 Datasheet Version 1.4b.  It is being used  //
// specifically with Crystal Fontz LCD CRC Check bits.										 //
//                                                                                           //
// The only modifications made to the original code were the data type descriptors.  The     //
// function declaration from "word" to "int32", "register unsgined int" to "int32", and      //
// "ubyte" to "unsigned char".  These are CCS PCWH compiler datatypes.						 //
//                                                                                           //
// For any questions regarding the legality of this document please contact					 // 
// mleonardi@futurec.net or Crystal Fontz at techinfo@crystalfontz.com...					 //
//																							 //
// The following is what Crystal Fontz had to say about this document and the code offered	 //
// under GPL.																				 //
///////////////////////////////////////////////////////////////////////////////////////////////
// This algorithm was mainly written to avoid any possible legal issues about the source of	 //
// the routine (at the request of the LCDproc group).  This routine was "clean" coded from	 //
// the definition of the CRC.  It is ostensibly smaller than the table driven approach but   //
// will take longer to execute.  This routine is offered under the GPL.                      //
///////////////////////////////////////////////////////////////////////////////////////////////

int32 get_crc(unsigned char *bufptr, int16 len)
{
int32
newCRC;
//Put the current byte in here.
unsigned char
data;
int
bit_count;
//This seed makes the output of this shift based algorithm match
//the table based algorithm. The center 16 bits of the 32-bit
//"newCRC" are used for the CRC. The MSb of the lower byte is used
//to see what bit was shifted out of the center 16 bit CRC
//accumulator ("carry flag analog");
newCRC=0x00F32100;
while(len--)
{
//Get the next byte in the stream.
data=*bufptr++;
//Push this bytes bits through a software
//implementation of a hardware shift & xor.
for(bit_count=0;bit_count<=7;bit_count++)
{
//Shift the CRC accumulator
newCRC>>=1;
//The new MSB of the CRC accumulator comes
//from the LSB of the current data byte.
if(data&0x01)
newCRC|=0x00800000;
//If the low bit of the current CRC accumulator was set
//before the shift, then we need to XOR the accumulator
//with the polynomial (center 16 bits of 0x00840800)
if(newCRC&0x00000080)
newCRC^=0x00840800;
//Shift the data byte to put the next bit of the stream
//into position 0.
data>>=1;
}
}
//All the data has been done. Do 16 more bits of 0 data.
for(bit_count=0;bit_count<=15;bit_count++)
{
//Shift the CRC accumulator
newCRC>>=1;
//If the low bit of the current CRC accumulator was set
//before the shift we need to XOR the accumulator with
//0x00840800.
if(newCRC&0x00000080)
newCRC^=0x00840800;
}
//Return the center 16 bits, making this CRC match the ones
//complement that is sent in the packet.
return((~newCRC)>>8);
}
